"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
var ValidateService_1;
Object.defineProperty(exports, "__esModule", { value: true });
exports.ValidateService = void 0;
const common_1 = require("@nestjs/common");
const EnergyTransferRequest_repository_1 = require("./repositories/EnergyTransferRequest.repository");
const cqrs_1 = require("@nestjs/cqrs");
const common_2 = require("@nestjs/common");
const ValidateTransferCommand_1 = require("./commands/ValidateTransferCommand");
const EnergyTransferRequest_1 = require("./EnergyTransferRequest");
const configuration_1 = require("./batch/configuration");
const events_1 = require("./batch/events");
let ValidateService = ValidateService_1 = class ValidateService {
    constructor(etrRepository, validateCommands, batchConfiguration, commandBus, eventBus) {
        this.etrRepository = etrRepository;
        this.validateCommands = validateCommands;
        this.batchConfiguration = batchConfiguration;
        this.commandBus = commandBus;
        this.eventBus = eventBus;
        this.logger = new common_2.Logger(ValidateService_1.name);
    }
    async validateTask() {
        const etrs = await this.etrRepository.findByState(EnergyTransferRequest_1.State.ValidationAwaiting, {
            limit: this.batchConfiguration.validateBatchSize
        });
        await this.startValidation(etrs);
        this.eventBus.publish(new events_1.AwaitingValidationEvent());
    }
    async startValidation(etrs) {
        etrs.forEach((etr) => etr.startValidation(this.validateCommands.map((c) => c.name)));
        await this.etrRepository.saveManyInTransaction(etrs);
        if (this.validateCommands.length === 0) {
            this.eventBus.publish(new events_1.AwaitingTransferEvent());
            return;
        }
        const validateEtrsPromises = etrs.map(async (etr) => {
            try {
                await this.validateEtr(etr);
            }
            catch (e) {
                etr.validationError(e.message);
                await this.etrRepository.save(etr);
            }
        });
        await Promise.all(validateEtrsPromises);
    }
    async updateValidationStatus(command) {
        const { requestId, validatorName: commandName, status } = command;
        let isValid = false;
        await this.etrRepository.updateWithLock(requestId, (request) => {
            const result = request.updateValidationStatus(commandName, status);
            switch (result) {
                case EnergyTransferRequest_1.UpdateStatusCode.NoValidator:
                    throw new Error(`Cannot update status of transfer request: ${requestId}, because validator "${commandName}" is not present`);
                case EnergyTransferRequest_1.UpdateStatusCode.NotPending:
                    this.logger.warn(`Skipping update of transfer request: ${requestId} to ${status}, because it is not pending`);
                    break;
                case EnergyTransferRequest_1.UpdateStatusCode.Success:
                    isValid = true;
                    break;
            }
        });
        if (isValid) {
            this.eventBus.publish(new events_1.AwaitingTransferEvent());
        }
    }
    async validateEtr(request) {
        const validationPromises = this.validateCommands.map(async (Command) => {
            const result = await this.executeCommand(Command, request);
            await this.updateValidationStatus({
                validatorName: Command.name,
                requestId: request.id,
                status: result.validationResult
            });
        });
        await Promise.all(validationPromises);
    }
    async executeCommand(Command, request) {
        try {
            const command = new Command(request.toPublicAttrs());
            return await this.commandBus.execute(command);
        }
        catch (e) {
            this.logger.error(`One of validation commands (${Command.name}) returned error for request: ${request.id}: "${e.message}"`);
            return { validationResult: EnergyTransferRequest_1.TransferValidationStatus.Error };
        }
    }
};
ValidateService = ValidateService_1 = __decorate([
    common_1.Injectable(),
    __param(0, common_1.Inject(EnergyTransferRequest_repository_1.ENERGY_TRANSFER_REQUEST_REPOSITORY)),
    __param(1, common_1.Inject(ValidateTransferCommand_1.VALIDATE_TRANSFER_COMMANDS_TOKEN)),
    __param(2, common_1.Inject(configuration_1.BATCH_CONFIGURATION_TOKEN)),
    __metadata("design:paramtypes", [Object, Array, Object, cqrs_1.CommandBus,
        cqrs_1.EventBus])
], ValidateService);
exports.ValidateService = ValidateService;
//# sourceMappingURL=validate.service.js.map